@@ -57,19 +57,28 @@ class NotesDB(utils.SubjectMixin):
         
         # we'll use this to store which notes are currently being synced by
         # the background thread, so we don't add them anew if they're still
-        # in progress
+        # in progress. This variable is only used by the background thread.
         self.threaded_syncing_keys = {}
         
+        # reading a variable or setting this variable is atomic
+        # so sync thread will write to it, main thread will only
+        # check it sometimes.
+        self.waiting_for_simplenote = False
+        
         # save and sync queue
-        # we only want ONE thread to do both saving and syncing
-        self.q_ss = Queue()
-        # but separate result queues for saving and syncing
+        self.q_save = Queue()
         self.q_save_res = Queue()
+
+        thread_save = Thread(target=self.worker_save)
+        thread_save.setDaemon(True)
+        thread_save.start()
+        
+        self.q_sync = Queue()
         self.q_sync_res = Queue()
         
-        thread_ss = Thread(target=self.worker_ss)
-        thread_ss.setDaemon(True)
-        thread_ss.start()
+        thread_sync = Thread(target=self.worker_sync)
+        thread_sync.setDaemon(True)
+        thread_sync.start()
         
     def create_note(self, title):
         # need to get a key unique to this database. not really important
@@ -148,9 +157,13 @@ class NotesDB(utils.SubjectMixin):
             o.synced = True
             
         return o
+
+    def get_save_queue_len(self):
+        return self.q_save.qsize()
+
             
-    def get_ss_queue_len(self):
-        return self.q_ss.qsize()
+    def get_sync_queue_len(self):
+        return self.q_sync.qsize()
         
     def helper_key_to_fname(self, k):
         return os.path.join(self.db_path, k) + '.json'
@@ -230,7 +243,7 @@ class NotesDB(utils.SubjectMixin):
                 cn = copy.deepcopy(n)
                 # put it on my queue as a save
                 o = utils.KeyValueObject(action=ACTION_SAVE, key=k, note=cn)
-                self.q_ss.put(o)
+                self.q_save.put(o)
                 
         # in this same call, we process stuff that might have been put on the result queue
         nsaved = 0
@@ -252,11 +265,26 @@ class NotesDB(utils.SubjectMixin):
         return nsaved
         
     
-    def sync_to_server_threaded(self):
+    def sync_to_server_threaded(self, wait_for_idle=True):
         """Only sync notes that have been changed / created locally since previous sync.
         
+        This function is called by the housekeeping handler, so once every
+        few seconds.
+        
+        @param wait_for_idle: Usually, last modification date has to be more
+        than a few seconds ago before a sync to server is attempted. If
+        wait_for_idle is set to False, no waiting is applied. Used by exit
+        cleanup in controller.
+        
         """
         
+        # this many seconds of idle time (i.e. modification this long ago)
+        # before we try to sync.
+        if wait_for_idle:
+            lastmod = 3
+        else:
+            lastmod = 0
+        
         now = time.time()
         for k,n in self.notes.items():
             # if note has been modified sinc the sync, we need to sync.
@@ -266,7 +294,7 @@ class NotesDB(utils.SubjectMixin):
             modifydate = float(n.get('modifydate', -1))
             syncdate = float(n.get('syncdate', -1))
             if modifydate > syncdate and \
-               now - modifydate > 3 and \
+               now - modifydate > lastmod and \
                k not in self.threaded_syncing_keys:
                 # record that we've requested a sync on this note,
                 # so that we don't keep on putting stuff on the queue.
@@ -276,7 +304,7 @@ class NotesDB(utils.SubjectMixin):
                 cn['syncdate'] = time.time()
                 # put it on my queue as a sync
                 o = utils.KeyValueObject(action=ACTION_SYNC_PARTIAL_TO_SERVER, key=k, note=cn)
-                self.q_ss.put(o)
+                self.q_sync.put(o)
                 
         # in this same call, we read out the result queue
         nsynced = 0
@@ -435,9 +463,9 @@ class NotesDB(utils.SubjectMixin):
             n['modifydate'] = time.time()
             self.notify_observers('change:note-status', utils.KeyValueObject(what='modifydate', key=key))
 
-    def worker_ss(self):
+    def worker_save(self):
         while True:
-            o = self.q_ss.get()
+            o = self.q_save.get()
             
             if o.action == ACTION_SAVE:
                 # this will write the savedate into o.note
@@ -450,8 +478,15 @@ class NotesDB(utils.SubjectMixin):
                 # somebody has to read out the queue...
                 self.q_save_res.put(o)
                 
-            elif o.action == ACTION_SYNC_PARTIAL_TO_SERVER:
+    def worker_sync(self):
+        while True:
+            o = self.q_sync.get()
+            
+            if o.action == ACTION_SYNC_PARTIAL_TO_SERVER:
+                self.waiting_for_simplenote = True
                 uret = self.simplenote.update_note(o.note)
+                self.waiting_for_simplenote = False
+                
                 if uret[1] == 0:
                     # success!
                     n = uret[0]
@@ -477,5 +512,5 @@ class NotesDB(utils.SubjectMixin):
                     
                 else:
                     o.error = 1
-                    
-                   
+                    self.q_sync_res.put(o)
+                    
\ No newline at end of file
