@@ -192,7 +192,7 @@ class Controller:
             return None
                 
     def get_version(self):
-        return "0.7.1"
+        return "0.8"
     
     def main_loop(self):
         self.view.main_loop()
@@ -316,17 +316,32 @@ class Controller:
         fn_uri = 'file://' + os.path.abspath(fn)
         webbrowser.open(fn_uri)
         
+    def helper_save_sync_msg(self):
+        # Saving 2 notes. Syncing 3 notes, waiting for simplenote server.
+        # All notes saved. All notes synced.
+        
+        saven = self.notes_db.get_save_queue_len()
+        syncn = self.notes_db.get_sync_queue_len()
+        wfsn = self.notes_db.waiting_for_simplenote
+        
+        savet = 'Saving %d notes.' % (saven,) if saven > 0 else '';
+        synct = 'Waiting to sync %d notes.' % (syncn,) if syncn > 0 else '';
+        wfsnt = 'Syncing with simplenote server.' if wfsn else '';
+        
+        return ' '.join([i for i in [savet, synct, wfsnt] if i])
+        
+        
     def observer_view_keep_house(self, view, evt_type, evt):
         # queue up all notes that need to be saved
         nsaved = self.notes_db.save_threaded()
         nsynced, sync_errors = self.notes_db.sync_to_server_threaded()
         
-        # get list of note titles, and pass to view to check and fix if necessary
-        qlen = self.notes_db.get_ss_queue_len() 
-        if qlen > 0:
-            self.view.set_status_text('Saving and syncing, %d notes in the queue.' % (qlen,))
-        else:
-            self.view.set_status_text('Idle.')
+        msg = self.helper_save_sync_msg()
+
+        if sync_errors:
+            msg = ' '.join([i for i in [msg, 'Could not connect to simplenote server.'] if i])
+            
+        self.view.set_status_text(msg)
 
         # in continous rendering mode, we also generate a new HTML
         # the browser, if open, will refresh!
@@ -368,6 +383,12 @@ class Controller:
 
         if idx < 0:
             self.view.select_note(0)
+            # the user is typing, but her previously selected note is
+            # not in the new filtered list. as a convenience, we move
+            # the text in the text widget so it's on the first
+            # occurrence of the search string, IF there's such an
+            # occurrenc.
+            self.view.see_first_search_instance()
             
         else:
             # we don't want new text to be implanted so we keep this silent
@@ -388,9 +409,27 @@ class Controller:
                                            self.view.get_text())
             
     def observer_view_close(self, view, evt_type, evt):
-        # do a last full sync before we go!
-        #self.sync_full()
-        pass
+        # check that everything has been saved and synced before exiting
+        
+        # first make sure all our queues are up to date
+        self.notes_db.save_threaded()
+        self.notes_db.sync_to_server_threaded(wait_for_idle=False)        
+        
+        # then check all queues
+        saven = self.notes_db.get_save_queue_len()
+        syncn = self.notes_db.get_sync_queue_len()
+        wfsn = self.notes_db.waiting_for_simplenote
+        
+        # if there's still something to do, warn the user.
+        if saven or syncn or wfsn:
+            msg = "Are you sure you want to exit? I'm still busy: " + self.helper_save_sync_msg()
+            really_want_to_exit = self.view.askyesno("Confirm exit", msg)
+            
+            if really_want_to_exit:
+                self.view.close()
+                
+        else:
+            self.view.close()
         
     def observer_view_create_note(self, view, evt_type, evt):
         # create the note
