@@ -169,6 +169,9 @@ class View(utils.SubjectMixin):
 
         self.search_entry.focus_set()
         
+    def askyesno(self, title, msg):
+        return tkMessageBox.askyesno(title, msg)
+    
     def cmd_lb_notes_select(self, evt):
         sidx = self.get_selected_idx()
         self.notify_observers('select:note', utils.KeyValueObject(sel=sidx))
@@ -222,13 +225,33 @@ class View(utils.SubjectMixin):
         
         #self.text_note.mark_set(tk.INSERT, cursor_pos)
 
+    def see_first_search_instance(self):
+        """If there are instances of the search string in the current
+        note, ensure that the first one is visible.
+        """
+
+        if self.text_tags_search:
+            self.text_note.see(self.text_tags_search[0] + '.first')
+
     def select_note(self, idx, silent=False):
-        # programmatically select the note by idx
+        """Programmatically select the note by idx
+
+        @param silent: If this is True, don't fire an event. VERY
+        IMPORTANT: if you use silent, the controller won't set the
+        selected_note_idx. You should make sure that it's in sync with
+        what you've just selected.
+        """
+
         self.lb_notes.select_clear(0, tk.END)
         self.lb_notes.select_set(idx)
         # we move the active (underlined) selection along, else we lose
         # synchronization during arrow movements with the search entry selected
         self.lb_notes.activate(idx)
+        
+        # make sure the selected index is visible
+        # this only moves the view if the selected note is really not visible
+        self.lb_notes.see(idx)
+        
         if not silent:
             # we have to generate event explicitly, it doesn't fire by itself in this case
             self.lb_notes.event_generate('<<ListboxSelect>>')
@@ -276,7 +299,7 @@ class View(utils.SubjectMixin):
         
     def _bind_events(self):
         # make sure window close also goes through our handler
-        self.root.protocol('WM_DELETE_WINDOW', self.close)
+        self.root.protocol('WM_DELETE_WINDOW', self.handler_close)
         
         self.lb_notes.bind("<<ListboxSelect>>", self.cmd_lb_notes_select)
         # same behaviour as when the user presses enter on search entry:
@@ -360,8 +383,8 @@ class View(utils.SubjectMixin):
         file_menu.add_separator()
 
         file_menu.add_command(label = "Exit", underline=1,
-                              command=self.cmd_exit, accelerator="Ctrl+Q")
-        self.root.bind_all("<Control-q>", self.cmd_exit)
+                              command=self.handler_close, accelerator="Ctrl+Q")
+        self.root.bind_all("<Control-q>", self.handler_close)
 
         # EDIT ##########################################################
         edit_menu = tk.Menu(menu, tearoff=False)
@@ -458,11 +481,18 @@ class View(utils.SubjectMixin):
         left_frame = tk.Frame(paned_window, width=100)
         paned_window.add(left_frame)
        
+        # setup the scrollbar
+        self.sb_notes = tk.Scrollbar(left_frame, orient=tk.VERTICAL)
+        
         # exportselection=0 means it doesn't automatically export to
         # x selection. with that active, selecting in the text widget
         # removes selection in listbox.
         # thank you http://stackoverflow.com/a/756875
-        self.lb_notes = tk.Listbox(left_frame, exportselection=0)
+        self.lb_notes = tk.Listbox(left_frame, exportselection=0,
+                                   yscrollcommand=self.sb_notes.set)
+        
+        self.sb_notes.config(command=self.lb_notes.yview)
+        self.sb_notes.pack(side=tk.RIGHT, fill=tk.Y)
         
         # need both fill and expand to make it fill all avail area
         self.lb_notes.pack(fill=tk.BOTH, expand=1)
@@ -531,11 +561,18 @@ class View(utils.SubjectMixin):
         self.root.update_idletasks()
         
         
-    def close(self):
+    def handler_close(self, evt=None):
+        """Handler for exit menu command and close window event.
+        """
         self.notify_observers('close', None)
+        
+    def close(self):
+        """Programmatically close application windows.
+        
+        Called by controller. 
+        """
         self.root.destroy()
 
-
     def cmd_cut(self):
         self.text_note.event_generate('<<Cut>>')
 
@@ -557,9 +594,6 @@ class View(utils.SubjectMixin):
             'A rather ugly but cross-platform simplenote client.' % (self.config.app_version,),
             parent = self.root)
 
-    def cmd_exit(self, event=None):
-        self.close()
-
     def cmd_rest(self, event=None):
         self.notify_observers('command:rest', None)
 
